#!/usr/bin/python
# -*- coding: utf-8 -*-
# nautilus-pastebin - Nautilus extension to paste a file to a pastebin service
# Written by:
#    Alessio Treglia <quadrispro@ubuntu.com>
# Copyright (C) 2009, Alessio Treglia
#
# This package is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This package is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this package; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

import gtk
import os, sys
from pastebin.core import GlobalConfigurationValidator, USERCONFIG_FILENAME
from ConfigParser import ConfigParser

def getSearchResPaths():
    return [
        os.path.join(sys.prefix, "share", "nautilus-pastebin"),
        os.path.join(sys.prefix, "share", "pixmaps"),
        os.path.join(sys.prefix, "local", "share", "nautilus-pastebin"),
        os.path.expanduser(os.path.join("~", ".nautilus-pastebin", "data")),
        os.getcwd()
    ]

def getPath(path):
    search_paths = getSearchResPaths()
    for search in search_paths:
        fullpath = os.path.join(search, path)
        if os.path.exists(fullpath):
            return fullpath
    else:
        raise IOError(_("Can't find %s in any known prefix!") % kwargs[component])

GLADE_FILE = getPath('nautilus-pastebin-configurator.glade')
ICON_FILE = getPath('nautilus-pastebin.png')
USERCONFIG_DIR = os.path.expanduser(os.path.join('~', '.config', 'nautilus-pastebin'))

class GlobalConfigurationValidator(GlobalConfigurationValidator):
    NEEDED_OPTS = GlobalConfigurationValidator.NEEDED_OPTS.extend(['enable_confirmation', 'enable_notification','enable_open_browser'])
    pass

class ConfigurationPersistence(object):

    def __init__(self):
        self.validator = GlobalConfigurationValidator()
        self.globals = {}
        self.pastebins = []

    def loadConfiguration(self):
        if self.validator.validate():
            self.globals = self.validator.globals
            self.pastebins = self.validator.getAvailablePastebins()
        else:
            raise Exception, _("Unable to find a valid configuration file.")

    def updateConfiguration(self):
        gcv = GlobalConfigurationValidator(files=[USERCONFIG_FILENAME])
        cp = gcv.getConfigParser()
        try:
            cp.add_section(GlobalConfigurationValidator.MAIN_SECTION)
        except:
            pass    # the section already exists
        for k in self.globals.keys():
            cp.set(GlobalConfigurationValidator.MAIN_SECTION, k, self.globals[k])
        
        confdir = os.path.dirname(USERCONFIG_FILENAME)
        if not os.path.exists(confdir):
            os.mkdir(confdir)
        if not os.path.isdir(confdir):
            raise Exception, _("Unable to open %s, another file with the same \
name already exists.") % confdir
        fp = open(USERCONFIG_FILENAME, 'w+')
        cp.write(fp)
        fp.close()

class Controller(object):

    def _retrieveConfigFromUI(self):
        params = self.confPers.globals.keys()
        for i in params:
            if i == 'pastebin':
                self.confPers.globals['pastebin'] = self.builder.get_object("combobox_pastebin").get_active_text()
            else:
                self.confPers.globals[i] = self.builder.get_object(i).get_active()

    def on_button_close_clicked(self, widget, data=None):
        self.dialogMain.destroy()
        self._retrieveConfigFromUI()
        try:
            self.confPers.updateConfiguration()
        except Exception, e:
            self.showErrorMessage(str(e))
        self.quit()

    def _setup_widgets(self):
        builder = self.builder
        try:
            self.confPers.loadConfiguration()
        except Exception, e:
            self.showErrorMessage(str(e))
            self.quit()
        generals = self.confPers.globals
        pastebins = self.confPers.pastebins
        # Set the icon
        self.dialogMain.set_icon_from_file(ICON_FILE)
        # Populate the combobox widget
        comboboxPastebin = builder.get_object("combobox_pastebin")
        comboboxPastebin.set_model(gtk.ListStore(str))
        cell = gtk.CellRendererText()
        comboboxPastebin.pack_start(cell, True)
        comboboxPastebin.add_attribute(cell, 'text', 0)

        [comboboxPastebin.append_text(k) for k in pastebins]
        comboboxPastebin.set_active(pastebins.index(generals['pastebin']))
        
        # Set the boolean flags
        for k in [i for i in generals.keys() if i.startswith('enable_')]:
            builder.get_object(k).set_active(generals[k])

    def __init__(self):
        builder = gtk.Builder()
        builder.add_from_file(GLADE_FILE)
        self.confPers = ConfigurationPersistence()
        self.dialogMain = builder.get_object('dialog_main')
        self.tableOptions = builder.get_object('table_options')
        builder.connect_signals(self)
        self.dialogMain.connect('delete_event', self.quit)
        
        self.builder = builder
        
        self._setup_widgets()
        self.dialogMain.show_all()
    def showErrorMessage(self, message):
        md = gtk.MessageDialog(parent=self.dialogMain,
            message_format=message,
            buttons=gtk.BUTTONS_CLOSE,
            type = gtk.MESSAGE_ERROR,
            flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        md.run()
        md.destroy()
    def quit(self, *args):
        gtk.main_quit()
           
        
def main():
    c = Controller()
    gtk.main()
    
if __name__ == "__main__":
    main()

